#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 by VladX, <http://vladx.net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.

import os
import pycurl
import xml.dom.minidom
import pygtk
pygtk.require('2.0')
import gtk
from hashlib import md5
from Imagehosting import ImageHosting, nanoshot

try:
	import gnome
	openurl = gnome.url_show
except:
	import webbrowser
	openurl = webbrowser.open

class Flickr(ImageHosting):
	
	HOSTING_NAME = 'Flickr'
	HOSTING_ICON = nanoshot.NANOSHOT_HOSTINGS_DIR + '/icons/Flickr.png'
	LOCALE_DOMAIN = 'Flickr'
	AUTHOR = 'VladX'
	VERSION = 0.1
	
	API_KEY = 'a43662b0cbf868869b109aa32d839e95'
	API_SECRET = '6fe07a0fbc53159c'
	
	def gen_sig(self, args):
		
		args['api_key'] = self.API_KEY
		data = [self.API_SECRET]
		keys = args.keys()
		keys.sort()
		for key in keys:
			data.append(key)
			data.append(str(args[key]))
		data = ''.join(data)
		return md5(data).hexdigest()
	
	def api_get(self, **args):
		
		args['api_sig'] = self.gen_sig(args)
		query_string = ''
		for arg in args.keys():
			query_string += arg + '=' + args[arg] + '&'
		curl = self.curl
		curl.setopt(curl.HTTPGET, 1)
		curl.setopt(curl.FOLLOWLOCATION, 1)
		curl.setopt(curl.URL, 'http://www.flickr.com/services/rest/?' + str(query_string))
		self.response = ''
		try:
			curl.perform()
		except pycurl.error:
			self.error('Can\'t send request to the server. Perhaps the server is down, or you have a problem with Internet connection.')
		dom = xml.dom.minidom.parseString(self.response)
		try:
			error = dom.getElementsByTagName('err')[0].getAttribute('msg')
		except:
			error = None
		if error == 'Invalid frob':
			self.error('You haven\'t confirmed authorization. Nanoshot can\'t upload a screenshot on Flickr.')
		elif error:
			self.error('The server returned an error: %s.' % error)
		self.response = ''
		return dom
	
	def store_token(self):
		
		if self.authToken:
			try:
				f = open(self.tokenFile, 'w')
				f.write(self.authToken)
				f.colse()
			except: pass
	
	def load_token(self):
		
		if os.path.isfile(self.tokenFile):
			token = None
			try:
				f = open(self.tokenFile, 'r')
				token = f.read().strip()
				f.close()
			except: pass
			return token
	
	def preparation(self):
		
		def response(dialog, responseId):
			
			dialog.destroy()
			if responseId == gtk.RESPONSE_OK:
				dom = self.api_get(method = 'flickr.auth.getToken', frob = frob)
				if self.errorText:
					return False
				token = dom.getElementsByTagName('token')
				if token:
					token = str(token[0].firstChild.nodeValue)
				perms = dom.getElementsByTagName('perms')
				if perms:
					perms = str(perms[0].firstChild.nodeValue)
				if not token or perms != 'write':
					self.error('Authorization failed!')
				self.authToken = token
			else:
				self.errorText = 'Aborted by user'
		
		self.tokenFile = os.path.dirname(nanoshot.nanoshot.configStoreFile) + '/flickr.token'
		self.authToken = None
		self.response = ''
		self.progressStr = ''
		self.progress = 0.0
		self.png = self.pixbuf2png()
		self.curl = curl = pycurl.Curl()
		curl.setopt(curl.USERAGENT, 'Mozilla/5.0 Nanoshot')
		curl.setopt(curl.WRITEFUNCTION, self.write)
		loadedToken = self.load_token()
		if loadedToken:
			perms = self.api_get(method = 'flickr.auth.checkToken', auth_token = loadedToken).getElementsByTagName('perms')
			self.errorText = ''
			if perms and perms[0].firstChild.nodeValue == 'write':
				self.authToken = loadedToken
		if not self.authToken:
			tags = self.api_get(method = 'flickr.auth.getFrob').getElementsByTagName('frob')
			if tags:
				frob = tags[0].firstChild.nodeValue
			else:
				self.error('An error occurred while trying to authorize.')
			openurl('http://www.flickr.com/services/auth/?api_key=' + self.API_KEY + '&perms=write&frob=' + frob + '&api_sig=' + self.gen_sig({'perms': 'write', 'frob': frob}))
			dialog = gtk.MessageDialog(buttons = gtk.BUTTONS_OK_CANCEL, message_format = 'This program requires your authorization before it can upload a screenshot on Flickr.')
			dialog.format_secondary_text('Go into your web browser and confirm the authorization. When you\'re finished, return to this window to complete authorization and click «OK».')
			dialog.connect('response', response)
			dialog.set_keep_above(True)
			dialog.run()
			if not self.authToken:
				return False
			self.store_token()
		args = {'tags': 'nanoshot', 'content_type': '2', 'auth_token': self.authToken}
		args['api_sig'] = self.gen_sig(args)
		args = args.items()
		args.append(('photo', (pycurl.FORM_FILE, self.png.name, pycurl.FORM_CONTENTTYPE, 'image/png')))
		curl.setopt(curl.URL, 'http://api.flickr.com/services/upload/')
		curl.setopt(curl.POST, 1)
		curl.setopt(curl.HTTPPOST, args)
		curl.setopt(curl.NOPROGRESS, 0)
		curl.setopt(curl.PROGRESSFUNCTION, self.set_progress)
	
	def upload(self):
		
		try:
			self.curl.perform()
		except pycurl.error:
			self.error('Uploading has failed. Perhaps the server is down, or you have a problem with Internet connection.')
		self.png.close()
		dom = xml.dom.minidom.parseString(self.response)
		try:
			error = dom.getElementsByTagName('err')[0].getAttribute('msg')
		except:
			error = None
		if error:
			self.error('The server returned an error: %s.' % error)
		photoId = dom.getElementsByTagName('photoid')
		if photoId:
			self.photoId = photoId[0].firstChild.nodeValue
		else:
			self.photoId = False
			self.error('Failed to upload a screenshot.')
		return True
	
	def display(self):
		
		if not self.photoId:
			return False
		dom = self.api_get(method = 'flickr.photos.getSizes', photo_id = self.photoId, auth_token = self.authToken)
		elems = dom.getElementsByTagName('size')
		table = gtk.Table(len(elems), 3)
		table.set_row_spacings(5)
		table.set_col_spacings(10)
		clipboard = gtk.Clipboard()
		row = 0
		for elem in elems:
			label = str(elem.getAttribute('label'))
			source = str(elem.getAttribute('source'))
			width = str(elem.getAttribute('width'))
			height = str(elem.getAttribute('height'))
			l = gtk.Label()
			l.set_markup('<b>' + self._(label) + '</b>' + ' (' + width + 'x' + height + ')')
			table.attach(l, 0, 1, row, row + 1)
			l = gtk.Label(source)
			l.set_selectable(True)
			alignment = gtk.Alignment(0, 0.5)
			alignment.add(l)
			table.attach(alignment, 1, 2, row, row + 1)
			button = gtk.Button(stock = gtk.STOCK_COPY)
			button.sourcetext = source
			button.connect('clicked', lambda button: clipboard.set_text(button.sourcetext))
			table.attach(button, 2, 3, row, row + 1)
			row += 1
		dialog = gtk.Dialog(self._('Image successfully uploaded'), None, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, (gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
		dialog.vbox.set_spacing(5)
		dialog.vbox.pack_start(table)
		dialog.vbox.show_all()
		dialog.connect('response', lambda d, r: d.destroy())
		dialog.run()
	
	def write(self, string):
		
		self.response += string
	
	def set_progress(self, downTotal, downCur, upTotal, upCur):
		
		upTotal = max(1, upTotal)
		self.progress = upCur / float(upTotal)
		self.progressStr = str(int(round(100 * self.progress))) + '%, ' + self.bytes2string(upCur) + ' ' + self._('of') + ' ' + self.bytes2string(upTotal)
	
	def get_progress(self):
		
		return self.progress, self.progressStr

Hosting = Flickr
