#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 by VladX, <http://vladx.net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.

import pycurl
import base64
import xml.dom.minidom
import pygtk
pygtk.require('2.0')
import gtk
from Imagehosting import ImageHosting, nanoshot

class ImageShack(ImageHosting):
	
	HOSTING_NAME = 'Imgur.com'
	HOSTING_ICON = nanoshot.NANOSHOT_HOSTINGS_DIR + '/icons/Imgur.png'
	LOCALE_DOMAIN = 'Imgur'
	AUTHOR = 'VladX'
	VERSION = 0.1
	
	def preparation(self):
		
		self.response = ''
		self.progressStr = ''
		self.progress = 0.0
		self.base64String = self.pixbuf2base64()
		self.curl = curl = pycurl.Curl()
		curl.setopt(pycurl.URL, 'http://api.imgur.com/2/upload.xml')
		curl.setopt(pycurl.USERAGENT, 'Mozilla/5.0 Nanoshot')
		curl.setopt(pycurl.POST, 1)
		curl.setopt(pycurl.HTTPPOST, [ ('key', '41218fddb9c47f6231b116fafdcfe71b'), ('image', self.base64String) ])
		curl.setopt(pycurl.WRITEFUNCTION, self.write)
		curl.setopt(curl.NOPROGRESS, 0)
		curl.setopt(curl.PROGRESSFUNCTION, self.set_progress)
	
	def upload(self):
		
		try:
			self.curl.perform()
		except pycurl.error:
			self.error('Uploading has failed. Perhaps the server is down, or you have a problem with Internet connection.')
		if self.curl.getinfo(pycurl.HTTP_CODE) != 200:
			self.error('Internal Error: server returned an invalid HTTP response code.')
		self.dom = xml.dom.minidom.parseString(self.response)
		try:
			error = self.dom.getElementsByTagName('error')[0].getElementsByTagName('message')[0].firstChild.nodeValue
		except:
			error = None
		if error:
			self.error(error)
		return True
	
	def display(self):
		
		def changed(combobox):
			
			index = combobox.get_active() - 1
			if index < 0:
				return False
			clipboard = gtk.Clipboard()
			clipboard.set_text(links[index][1])
		
		links = []
		linksText = {
						'original': 'Original size (direct link)',
						'small_square': 'Small square (direct link)',
						'large_thumbnail': 'Large thumbnail (direct link)'}
		for tag in ('original', 'small_square', 'large_thumbnail'):
			link = self.dom.getElementsByTagName(tag)
			if link:
				links.append((tag, link[0].firstChild.nodeValue))
		donePage = self.dom.getElementsByTagName('imgur_page')
		deletePage = self.dom.getElementsByTagName('delete_page')
		if donePage:
			donePage = donePage[0].firstChild.nodeValue
		if deletePage:
			deletePage = deletePage[0].firstChild.nodeValue
		dialog = gtk.Dialog(self._('Image successfully uploaded'), None, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, (gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
		combobox = gtk.combo_box_new_text()
		combobox.append_text(self._('Copy to clipboard:'))
		for tag, content in links:
			combobox.append_text(self._(linksText[tag]))
		combobox.set_active(0)
		combobox.connect('changed', changed)
		dialog.vbox.set_spacing(5)
		dialog.vbox.pack_start(combobox)
		if donePage:
			dialog.vbox.pack_start(gtk.LinkButton(donePage, self._('Go to the final page')))
		if deletePage:
			dialog.vbox.pack_start(gtk.LinkButton(deletePage, self._('Delete image')))
		dialog.vbox.show_all()
		dialog.connect('response', lambda d, r: d.destroy())
		dialog.run()
	
	def write(self, string):
		
		self.response += string
	
	def set_progress(self, downTotal, downCur, upTotal, upCur):
		
		upTotal = max(1, upTotal)
		self.progress = upCur / float(upTotal)
		self.progressStr = str(int(round(100 * self.progress))) + '%, ' + self.bytes2string(upCur) + ' ' + self._('of') + ' ' + self.bytes2string(upTotal)
	
	def get_progress(self):
		
		return self.progress, self.progressStr
	
	def pixbuf2base64(self):
		
		pngData = []
		self.pixbuf.save_to_callback(lambda data: pngData.append(data), 'png')
		pngData = ''.join(pngData)
		return base64.b64encode(pngData)

Hosting = ImageShack
