#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Adapted for Nanoshot by VladX <vvladxx@gmail.com>, (c) 2010
#
# Advene: Annotate Digital Videos, Exchange on the NEt
# Copyright (C) 2009 Olivier Aubert <olivier.aubert@liris.cnrs.fr>
#
# Advene is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Advene is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Advene; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import sys
import gobject
import gtk
import pygst
pygst.require('0.10')
import gst
import threading

gobject.threads_init()

class NotifySink(gst.BaseSink):
	
	__gsttemplates__ = (
		gst.PadTemplate('sink',
						gst.PAD_SINK,
						gst.PAD_ALWAYS,
						gst.caps_new_any()),
	)
	
	__gstdetails__ = ('Notify sink', 'Sink', 'Notifying sink', 'Olivier Aubert <olivier.aubert@liris.cnrs.fr>; modified by VladX <vvladxx@gmail.com>')
	
	__gproperties__ = { 'notify': ( gobject.TYPE_PYOBJECT, 'notify', 'The notify method', gobject.PARAM_READWRITE ) }
	
	def __init__(self):
		
		gst.BaseSink.__init__(self)
		self.set_sync(False)
		self._notify = None
	
	def do_preroll(self, buffer):
		
		if self._notify is not None:
			self._notify(buffer)
		return gst.FLOW_OK
	
	def do_set_property(self, key, value):
		
		if key.name == 'notify':
			self._notify=value
	
	def do_get_property(self, key):
		
		if key.name == 'notify':
			return self._notify


gst.element_register(NotifySink, 'notifysink')
gobject.type_register(NotifySink)


class Snapshotter(object):
	
	def __init__(self, uri, filename_prefix = '/tmp/frame_', *timestamps):
		
		if type(timestamps[0]).__name__ == 'list':
			self.timestamps = [0] + timestamps[0]
		else:
			self.timestamps = (0) + timestamps
		
		self.filename_prefix = filename_prefix
		self.snapshot_ready = True
		self.iter = -1
		
		videobin = gst.Bin()
		self.player = gst.element_factory_make('playbin')
		csp = gst.element_factory_make('ffmpegcolorspace')
		pngenc = gst.element_factory_make('pngenc')
		queue = gst.element_factory_make('queue')
		sink = gst.element_factory_make('notifysink')
		
		videobin.add(csp, pngenc, queue, sink)
		gst.element_link_many(csp, pngenc, queue, sink)
		
		videobin.add_pad(gst.GhostPad('sink', csp.get_pad('sink')))
		
		self.player.props.video_sink = videobin
		self.player.set_state(gst.STATE_NULL)
		self.player.props.uri = uri
		self.player.set_state(gst.STATE_PAUSED)
		
		bus = self.player.get_bus()
		bus.enable_sync_message_emission()
		
		sink.props.notify = self.queue_notify
	
	def snapshot(self, t):
		
		t = long(t * gst.SECOND)
		self.player.set_state(gst.STATE_PAUSED)
		res = self.player.seek_simple(gst.FORMAT_TIME, gst.SEEK_FLAG_FLUSH | gst.SEEK_FLAG_KEY_UNIT, t)
		if not res:
			print('Error when sending event')
		return True
	
	def queue_notify(self, buffer):
		
		if self.iter == 0:
			self.snapshot_ready = True
			return True
		fname = self.filename_prefix + str(self.iter) + '.png'
		f = open(fname, 'w')
		f.write(buffer.data)
		f.close()
		self.snapshot_ready = True
		return True
	
	def process_queue(self):
		
		while True:
			if not self.snapshot_ready:
				continue
			if not self.timestamps:
				break
			timestamp = long(self.timestamps.pop(0))
			self.iter += 1
			self.snapshot_ready = False
			self.snapshot(timestamp)
		return True
	
	def wait_for_completion(self):
		
		if self.snapshot_ready and not self.timestamps:
			self.loop.quit()
			return False
		return True
	
	def start(self):
		
		thread = threading.Thread(target = self.process_queue)
		thread.setDaemon(True)
		thread.start()
		self.loop = gobject.MainLoop()
		gobject.idle_add(self.wait_for_completion)
		self.loop.run()
		return True
