#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 by VladX, <http://vladx.net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.

import os
import gettext
import tempfile
import threading

nanoshot = os.sys.modules['__main__']
hostings = []

def load(directory):
	
	os.sys.path.insert(1, directory)
	names = os.listdir(directory)
	names.sort()
	for name in names:
		if not name.endswith('.py'):
			continue
		name = ''.join(name.split('.')[:-1])
		module = __import__(name)
		hostings.append(module)

class ImageHosting:
	
	HOSTING_NAME = 'Dummy'
	HOSTING_ICON = nanoshot.NANOSHOT_HOSTINGS_DIR + '/icons/default.png'
	LOCALE_DIR = nanoshot.NANOSHOT_HOSTINGS_DIR + '/locale'
	LOCALE_DOMAIN = None
	AUTHOR = 'Anon.'
	VERSION = 1.0
	
	def __init__(self, pixbuf):
		
		try:
			self._ = gettext.translation(self.LOCALE_DOMAIN, self.LOCALE_DIR).gettext
		except:
			self._ = lambda text: text
		self.pixbuf = pixbuf
		self.errorText = False
		self.preparation()
		if self.errorText:
			return None
		self._thread = threading.Thread(target = self.upload)
		self._thread.setDaemon(True)
		self._thread.start()
	
	def preparation(self):
		
		print('Method "preparation" is not declared.')
	
	def upload(self):
		
		print('Method "upload" is not declared.')
		return False
	
	def display(self):
		
		print('Method "display" is not declared.')
	
	def get_progress(self):
		
		print('Method "get_progress" is not declared.')
		return 0.0, ''
	
	def error(self, text = None):
		
		if text == None:
			return self.errorText
		self.errorText = self._(text)
		nanoshot.nanoshot.error(self.errorText)
	
	def bytes2string(self, bytes):
		
		p = 'B'
		bytes = float(bytes)
		if bytes >= 1024 ** 3:
			p = 'Gb'
			bytes = bytes / (1024 ** 3)
		elif bytes >= 1024 ** 2:
			p = 'Mb'
			bytes = bytes / (1024 ** 2)
		elif bytes >= 1024:
			p = 'Kb'
			bytes = bytes / 1024
		if p == 'B':
			bytes = str(int(bytes))
		else:
			bytes = '%.1f' % bytes
		return bytes + ' ' + self._(p)
	
	def pixbuf2png(self):
		
		png = tempfile.NamedTemporaryFile()
		self.pixbuf.save(png.name, 'png')
		return png
