#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 by VladX, <http://vladx.net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.

import pygtk
pygtk.require('2.0')
import gtk
import gobject
import cairo

class Selector:
	
	def __init__(self, eventHandler):
		
		self.selected = False
		self.makesSelection = False
		self.moving = False
		self.resizing = False
		root = gtk.gdk.get_default_root_window()
		self.screenWidth, self.screenHeight = root.get_size()
		width, height = self.screenWidth, self.screenHeight
		self.drawingWindow = drawingWindow = gtk.Window(gtk.WINDOW_POPUP)
		drawingWindow.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_SPLASHSCREEN)
		drawingWindow.set_decorated(False)
		drawingWindow.set_has_frame(False)
		drawingWindow.set_skip_taskbar_hint(True)
		drawingWindow.set_skip_pager_hint(True)
		drawingWindow.set_can_focus(True)
		drawingWindow.set_accept_focus(True)
		drawingWindow.set_keep_above(True)
		drawingWindow.set_default_size(width, height)
		drawingWindow.resize(width, height)
		drawingWindow.move(0, 0)
		rgba = drawingWindow.get_screen().get_rgba_colormap()
		drawingWindow.set_app_paintable(True)
		drawingWindow.set_colormap(rgba)
		drawingWindow.set_events(	gtk.gdk.BUTTON_MOTION_MASK | gtk.gdk.BUTTON1_MOTION_MASK | gtk.gdk.BUTTON2_MOTION_MASK |
									gtk.gdk.BUTTON3_MOTION_MASK | gtk.gdk.POINTER_MOTION_MASK | gtk.gdk.BUTTON_PRESS_MASK |
									gtk.gdk.BUTTON_RELEASE_MASK | gtk.gdk.KEY_PRESS_MASK | gtk.gdk.KEY_RELEASE_MASK
								)
		drawingWindow.show()
		drawingWindow.present()
		gtk.gdk.keyboard_grab(drawingWindow.window)
		cursor = gtk.gdk.Cursor(gtk.gdk.CROSSHAIR)
		drawingWindow.window.set_cursor(cursor)
		self.cursor = 'crosshair'
		drawingWindow.connect_after('expose-event', self.expose_event)
		drawingWindow.connect('button-press-event', eventHandler, self)
		drawingWindow.connect('button-release-event', eventHandler, self)
		drawingWindow.connect('key-press-event', eventHandler, self)
		drawingWindow.connect('key-release-event', eventHandler, self)
		drawingWindow.connect('motion-notify-event', eventHandler, self)
		if not hasattr(self, 'check_damaged_counter'):
			self.check_damaged_counter = 0
		if self.check_damaged_counter < 50:
			gobject.timeout_add(50, self.check_damaged, eventHandler)
	
	def __del__(self):
		
		if hasattr(self, 'deleted'):
			return True
		ctx = self.drawingWindow.window.cairo_create()
		self.clear(ctx)
		gtk.gdk.keyboard_ungrab()
		# Give cairo some time to clear the screen before the destruction of the window
		gobject.timeout_add(50, self.drawingWindow.destroy)
		if self.selected and hasattr(self, 'completeHandler'):
			x = int(round(min(self.selection_x_start, self.selection_x_end)))
			y = int(round(min(self.selection_y_start, self.selection_y_end)))
			width = int(round(abs(self.selection_x_end - self.selection_x_start)))
			height = int(round(abs(self.selection_y_end - self.selection_y_start)))
			gobject.timeout_add(150, self.completeHandler, x, y, width, height)
		self.deleted = True
	
	def start_selection(self, x, y):
		
		if self.makesSelection:
			return False
		self.selection_x_start = x
		self.selection_y_start = y
		self.selected = True
		self.makesSelection = True
		self.dark_fill()
	
	def stop_selection(self, x, y):
		
		if not self.makesSelection:
			return False
		self.selection_x_end = x
		self.selection_y_end = y
		self.makesSelection = False
		self.drawingWindow.present()
	
	def redraw(self, x, y):
		
		if not self.selected:
			return False
		rLefx = min(self.selection_x_start, x)
		rTopy = min(self.selection_y_start, y)
		rRigx = max(self.selection_x_start, x)
		rBoty = max(self.selection_y_start, y)
		width = rRigx - rLefx
		height = rBoty - rTopy
		ctx = self.drawingWindow.window.cairo_create()
		ctx.rectangle(rLefx - 50, rTopy - 50, width + 100, height + 100)
		ctx.clip()
		ctx.new_path()
		ctx.set_operator(cairo.OPERATOR_SOURCE)
		ctx.set_antialias(cairo.ANTIALIAS_NONE)
		ctx.set_source_rgba(0.0, 0.0, 0.0, 0.5)
		ctx.paint()
		ctx.set_source_rgba(0.0, 0.0, 255.0, 0.7)
		ctx.rectangle(rLefx, rTopy, width, height)
		ctx.fill()
		ctx.set_source_rgba(0.0, 0.0, 0.0, 0.0)
		ctx.rectangle(rLefx + 1, rTopy + 1, width - 2, height - 2)
		ctx.fill()
		if width > 36 and height > 36:
			halfWidth = round(width / 2)
			halfHeight = round(height / 2)
			ctx.set_source_rgba(0.0, 0.0, 255.0, 0.8)
			ctx.rectangle(rLefx, rTopy + halfHeight - 5, 10, 10)
			ctx.fill()
			ctx.set_source_rgba(0.0, 0.0, 255.0, 0.2)
			ctx.rectangle(rLefx + 1, rTopy + halfHeight - 4, 8, 8)
			ctx.fill()
			ctx.set_source_rgba(0.0, 0.0, 255.0, 0.8)
			ctx.rectangle(rLefx, rTopy, 10, 10)
			ctx.fill()
			ctx.set_source_rgba(0.0, 0.0, 255.0, 0.2)
			ctx.rectangle(rLefx + 1, rTopy + 1, 8, 8)
			ctx.fill()
			ctx.set_source_rgba(0.0, 0.0, 255.0, 0.8)
			ctx.rectangle(rLefx + halfWidth - 5, rTopy, 10, 10)
			ctx.fill()
			ctx.set_source_rgba(0.0, 0.0, 255.0, 0.2)
			ctx.rectangle(rLefx + halfWidth - 4, rTopy + 1, 8, 8)
			ctx.fill()
			ctx.set_source_rgba(0.0, 0.0, 255.0, 0.8)
			ctx.rectangle(rRigx - 10, rTopy, 10, 10)
			ctx.fill()
			ctx.set_source_rgba(0.0, 0.0, 255.0, 0.2)
			ctx.rectangle(rRigx - 9, rTopy + 1, 8, 8)
			ctx.fill()
			ctx.set_source_rgba(0.0, 0.0, 255.0, 0.8)
			ctx.rectangle(rLefx, rBoty - 10, 10, 10)
			ctx.fill()
			ctx.set_source_rgba(0.0, 0.0, 255.0, 0.2)
			ctx.rectangle(rLefx + 1, rBoty - 9, 8, 8)
			ctx.fill()
			ctx.set_source_rgba(0.0, 0.0, 255.0, 0.8)
			ctx.rectangle(rLefx + halfWidth - 5, rBoty - 10, 10, 10)
			ctx.fill()
			ctx.set_source_rgba(0.0, 0.0, 255.0, 0.2)
			ctx.rectangle(rLefx + halfWidth - 4, rBoty - 9, 8, 8)
			ctx.fill()
			ctx.set_source_rgba(0.0, 0.0, 255.0, 0.8)
			ctx.rectangle(rRigx - 10, rBoty - 10, 10, 10)
			ctx.fill()
			ctx.set_source_rgba(0.0, 0.0, 255.0, 0.2)
			ctx.rectangle(rRigx - 9, rBoty - 9, 8, 8)
			ctx.fill()
			ctx.set_source_rgba(0.0, 0.0, 255.0, 0.8)
			ctx.rectangle(rRigx - 10, rTopy + halfHeight - 5, 10, 10)
			ctx.fill()
			ctx.set_source_rgba(0.0, 0.0, 255.0, 0.2)
			ctx.rectangle(rRigx - 9, rTopy + halfHeight - 4, 8, 8)
			ctx.fill()
		# Show the selection size
		ctx.select_font_face('monospace', cairo.FONT_SLANT_NORMAL, cairo.FONT_WEIGHT_BOLD)
		ctx.set_font_size(11)
		text = str(int(width)) + 'x' + str(int(height))
		xBearing, yBearing, textWidth, textHeight = ctx.text_extents(text)[:4]
		if width > textWidth + 50 and height > textHeight + 50:
			textX, textY = rLefx - xBearing + round((width - textWidth) / 2.0), rTopy - yBearing + round((height - textHeight) / 2.0)
			ctx.rectangle(textX - 3 + xBearing, textY - 3 + yBearing, textWidth + 6, textHeight + 6)
			ctx.set_source_rgba(0.0, 0.0, 0.0, 0.8)
			ctx.fill()
			ctx.set_operator(cairo.OPERATOR_OVER)
			ctx.set_source_rgb(255.0, 255.0, 255.0)
			ctx.move_to(textX, textY)
			ctx.show_text(text)
	
	def start_move(self, x, y):
		
		if self.moving:
			return False
		self.moving = True
		self.moving_start_x = x
		self.moving_start_y = y
		self.drawingWindow.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.FLEUR))
		self.cursor = 'fleur'
	
	def stop_move(self):
		
		if not self.moving:
			return False
		self.moving = False
		self.drawingWindow.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.HAND1))
		self.cursor = 'hand1'
		self.drawingWindow.present()
	
	def move(self, x, y):
		
		if not self.moving:
			return False
		self.selection_x_start += x - self.moving_start_x
		self.selection_y_start += y - self.moving_start_y
		self.selection_x_end += x - self.moving_start_x
		self.selection_y_end += y - self.moving_start_y
		self.moving_start_x = x
		self.moving_start_y = y
		if self.selection_x_start < 0:
			self.selection_x_end -= self.selection_x_start
			self.selection_x_start = 0
		if self.selection_x_end < 0:
			self.selection_x_start -= self.selection_x_end
			self.selection_x_end = 0
		if self.selection_x_start > self.screenWidth:
			self.selection_x_end += self.screenWidth - self.selection_x_start
			self.selection_x_start = self.screenWidth
		if self.selection_x_end > self.screenWidth:
			self.selection_x_start += self.screenWidth - self.selection_x_end
			self.selection_x_end = self.screenWidth
		if self.selection_y_start < 0:
			self.selection_y_end -= self.selection_y_start
			self.selection_y_start = 0
		if self.selection_y_end < 0:
			self.selection_y_start -= self.selection_y_end
			self.selection_y_end = 0
		if self.selection_y_start > self.screenHeight:
			self.selection_y_end += self.screenHeight - self.selection_y_start
			self.selection_y_start = self.screenHeight
		if self.selection_y_end > self.screenHeight:
			self.selection_y_start += self.screenHeight - self.selection_y_end
			self.selection_y_end = self.screenHeight
		self.redraw(self.selection_x_end, self.selection_y_end)
	
	def move_up_down(self, keyval):
		
		if self.moving:
			return False
		self.moving = True
		self.moving_start_x = 0
		self.moving_start_y = 0
		step = int(round(self.screenWidth / 426.6))
		if keyval == gtk.gdk.keyval_from_name('Up'):
			x, y = 0, -step
		if keyval == gtk.gdk.keyval_from_name('Down'):
			x, y = 0, step
		if keyval == gtk.gdk.keyval_from_name('Left'):
			x, y = -step, 0
		if keyval == gtk.gdk.keyval_from_name('Right'):
			x, y = step, 0
		self.move(x, y)
		self.moving = False
	
	def start_resize(self, x, y):
		
		if self.resizing:
			return False
		self.resizing = True
		self.resizeDirection = self.pointer_on_the_resizing_square(x, y)
	
	def stop_resize(self):
		
		if not self.resizing:
			return False
		self.resizing = False
		self.drawingWindow.present()
	
	def resize(self, x, y):
		
		if not self.resizing:
			return False
		rLefx = min(self.selection_x_start, self.selection_x_end)
		rTopy = min(self.selection_y_start, self.selection_y_end)
		rRigx = max(self.selection_x_start, self.selection_x_end)
		rBoty = max(self.selection_y_start, self.selection_y_end)
		if self.resizeDirection == gtk.gdk.LEFT_SIDE:
			self.selection_x_end = max(0, x)
			self.selection_x_start = rRigx
		elif self.resizeDirection == gtk.gdk.TOP_SIDE:
			self.selection_y_end = max(0, y)
			self.selection_y_start = rBoty
		elif self.resizeDirection == gtk.gdk.RIGHT_SIDE:
			self.selection_x_end = max(0, x)
			self.selection_x_start = rLefx
		elif self.resizeDirection == gtk.gdk.BOTTOM_SIDE:
			self.selection_y_end = max(0, y)
			self.selection_y_start = rTopy
		elif self.resizeDirection == gtk.gdk.TOP_LEFT_CORNER:
			self.selection_x_end = max(0, x)
			self.selection_x_start = rRigx
			self.selection_y_end = max(0, y)
			self.selection_y_start = rBoty
		elif self.resizeDirection == gtk.gdk.TOP_RIGHT_CORNER:
			self.selection_x_end = max(0, x)
			self.selection_x_start = rLefx
			self.selection_y_end = max(0, y)
			self.selection_y_start = rBoty
		elif self.resizeDirection == gtk.gdk.BOTTOM_RIGHT_CORNER:
			self.selection_x_end = max(0, x)
			self.selection_x_start = rLefx
			self.selection_y_end = max(0, y)
			self.selection_y_start = rTopy
		elif self.resizeDirection == gtk.gdk.BOTTOM_LEFT_CORNER:
			self.selection_x_end = max(0, x)
			self.selection_x_start = rRigx
			self.selection_y_end = max(0, y)
			self.selection_y_start = rTopy
		if abs(self.selection_x_start - self.selection_x_end) <= 20 or abs(self.selection_y_start - self.selection_y_end) <= 20:
			return False
		self.dark_fill()
		self.redraw(self.selection_x_end, self.selection_y_end)
	
	def pointer_in_selection(self, x, y):
		
		rLefx = min(self.selection_x_start, self.selection_x_end)
		rTopy = min(self.selection_y_start, self.selection_y_end)
		rRigx = max(self.selection_x_start, self.selection_x_end)
		rBoty = max(self.selection_y_start, self.selection_y_end)
		if x >= rLefx and x <= rRigx and y >= rTopy and y <= rBoty:
			return True
		return False
	
	def pointer_on_the_resizing_square(self, x, y):
		
		rLefx = min(self.selection_x_start, self.selection_x_end)
		rTopy = min(self.selection_y_start, self.selection_y_end)
		rRigx = max(self.selection_x_start, self.selection_x_end)
		rBoty = max(self.selection_y_start, self.selection_y_end)
		halfWidth, halfHeight = (rRigx - rLefx) / 2, (rBoty - rTopy) / 2
		if (x >= rLefx and x <= rLefx + 5 and y > rTopy + 12 and y < rBoty - 12) or (x >= rLefx and x <= rLefx + 12 and y >= rTopy + halfHeight - 6 and y <= rTopy + halfHeight + 6):
			return gtk.gdk.LEFT_SIDE
		if (x > rLefx + 12 and x < rRigx - 12 and y >= rTopy and y <= rTopy + 5) or (x >= rLefx + halfWidth - 6 and x <= rLefx + halfWidth + 6 and y >= rTopy and y <= rTopy + 12):
			return gtk.gdk.TOP_SIDE
		if (x >= rRigx - 5 and x <= rRigx and y > rTopy + 12 and y < rBoty - 12) or (x >= rRigx - 12 and x <= rRigx and y >= rTopy + halfHeight - 6 and y <= rTopy + halfHeight + 6):
			return gtk.gdk.RIGHT_SIDE
		if (x > rLefx + 12 and x < rRigx - 12 and y >= rBoty - 5 and y <= rBoty) or (x >= rLefx + halfWidth - 6 and x <= rLefx + halfWidth + 6 and y >= rBoty - 12 and y <= rBoty):
			return gtk.gdk.BOTTOM_SIDE
		if x >= rLefx and x <= rLefx + 12 and y >= rTopy and y <= rTopy + 12:
			return gtk.gdk.TOP_LEFT_CORNER
		if x >= rRigx - 12 and x <= rRigx and y >= rTopy and y <= rTopy + 12:
			return gtk.gdk.TOP_RIGHT_CORNER
		if x >= rRigx - 12 and x <= rRigx and y >= rBoty - 12 and y <= rBoty:
			return gtk.gdk.BOTTOM_RIGHT_CORNER
		if x >= rLefx and x <= rLefx + 12 and y >= rBoty - 12 and y <= rBoty:
			return gtk.gdk.BOTTOM_LEFT_CORNER
		return False
	
	def cursor_handler(self, x, y):
		
		if not self.selected:
			return False
		if self.pointer_in_selection(x, y):
			cursor = self.pointer_on_the_resizing_square(x, y)
			if cursor:
				self.drawingWindow.window.set_cursor(gtk.gdk.Cursor(cursor))
				self.cursor = 'dest'
			elif self.cursor != 'hand1':
				self.drawingWindow.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.HAND1))
				self.cursor = 'hand1'
		elif self.cursor != 'crosshair':
			self.drawingWindow.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.CROSSHAIR))
			self.cursor = 'crosshair'
	
	def clear(self, ctx):
		
		ctx.set_operator(cairo.OPERATOR_CLEAR)
		ctx.rectangle(0, 0, self.screenWidth, self.screenHeight)
		ctx.fill()
		return ctx
	
	def dark_fill(self):
		
		ctx = self.drawingWindow.window.cairo_create()
		ctx.set_operator(cairo.OPERATOR_SOURCE)
		ctx.set_source_rgba(0.0, 0.0, 0.0, 0.5)
		ctx.paint()
	
	def expose_event(self, widget, event):
		
		self.dark_fill()
		if self.selected:
			#ctx = widget.window.cairo_create()
			#ctx = self.clear(ctx)
			self.redraw(self.selection_x_end, self.selection_y_end)
	
	def set_complete_handler(self, completeHandler):
		
		self.completeHandler = completeHandler
	
	def check_damaged(self, eventHandler):
		
		checkSize = self.drawingWindow.window.get_size()
		if checkSize[0] < 10 or checkSize[1] < 10:
			self.drawingWindow.destroy()
			self.check_damaged_counter += 1
			self.__init__(eventHandler)
		return False
